require 'net/http'
require 'json'
require 'nokogiri'

module IntelliJ
  module Helpers
    def get_all_releases(edition)
      e_code = node['intellij']['editions'][edition]
      url = node['intellij']['helpers']['releases'] % {
          edition: e_code,
          timestamp: Time.now.getutc
      }
      uri = URI(url)
      response = JSON.parse(Net::HTTP.get(uri))
      releases = response[e_code] || []
      result = {}
      releases.each do |release|
        result[release['version']] = release
      end
      result
    end

    def get_default_version(releases)
      releases.keys[0]
    end

    def get_release_data(version, edition)
      releases = get_all_releases(edition)
      version ||= get_default_version(releases)
      result = releases[version]
      Chef::Application.fatal!("The IntelliJ version #{version} of edition #{edition} could not be found!", 42) unless result
      result
    end

    def get_release_download_info(release)
      os = Chef::Platform.windows? ? 'windowsZip' : 'linux'
      info = release['downloads'][os]
      [info['link'], download_checksum_from_link(info['checksumLink'])]
    end

    def download_checksum_from_link(link)
      uri = URI(link)
      response = Net::HTTP.get(uri)
      response.split(' ').first
    end

    def find_plugin_suggestions(name, edition)
      product = node['intellij']['plugin']['product'][edition]
      url = node['intellij']['plugin']['search'] % { name: name, product: product }
      uri = URI(url)
      response = JSON.parse(Net::HTTP.get(uri))
      result = {}
      response.each do |suggestion|
        name = suggestion['value']
        id = suggestion['data']['url'].match(/\/plugin\/(\d+)-.*/)[1]
        result[name] = {}
        result[name]['id'] = id
        result[name]['name'] = name
        result[name]['family'] = suggestion['data']['target']
      end
      result
    end

    def find_plugin_updates(id, family)
      url = node['intellij']['plugin']['updates'] % { id: id }
      uri = URI(url)
      response = JSON.parse(Net::HTTP.get(uri))
      result = {}
      response['updates'].each do |update|
        link = node['intellij']['plugin']['download_link'] % {
            file: update['file'],
            update_id: update['id'],
            plugin_id: id,
            family: family
        }
        result[update['id']] = link
      end
      result
    end

    def detect_vcs(path)
      vcs_type = nil
      if ::File.exist?("#{path}/.git")
        vcs_type = :Git
      end
      vcs_type
    end

    def create_missing_component(name, file, fragment = nil, parent = 'project')
      xml_edit file do
        parent "/#{parent}"
        target "/#{parent}/component[@name=\"#{name}\"]"
        fragment "<component name=\"#{name}\">#{fragment}</component>"
        action :append_if_missing
        not_if { ::File.read(file).match(/<component.*name="#{name}".*>/) }
      end
    end

    def create_missing_file(file, source, user  = nil, group  = nil, variables = nil)
      directory ::File.dirname(file) do
        recursive true
        user user
        group group
      end

      template file do
        source source
        cookbook 'intellij'
        user user
        group group
        variables variables
        action :create_if_missing
      end
    end

    def process_xpath_in_xml(file, xpath)
      result = nil
      if ::File.exist?(file)
        doc = Nokogiri::XML(File.read(file), &:noblanks)
        namespace = doc.root.namespaces
        result = doc.xpath(xpath, namespace)
      end
      result
    end

    def pretty_format_xml(xml, indent = 2)
      Nokogiri::XML(xml, &:noblanks).root.to_xml(indent: indent)
    end

    def get_tmp_path(filename)
      "#{Dir.tmpdir}/#{::File.basename(filename)}"
    end
  end
end
