resource_name :intellij_module
default_action :create

property :path, String, name_property: true
property :type, Symbol, equal_to: [:JAVA_MODULE, :RUBY_MODULE, :WEB_MODULE], default: :WEB_MODULE
property :sdk_type, Symbol, equal_to: [:JavaSDK, :RUBY_SDK, :none], default: :none
property :sdk_name, String
property :module_name, String
property :project_path, String, required: true
property :delete_content, [TrueClass, FalseClass], default: false
property :user, String
property :group, String

include IntelliJ::Helpers
include IntelliJ::Module

action :create do
  begin
    args = @new_resource

    module_file = generate_module_file_path(args.path, args.module_name)

    directory ::File.dirname(module_file) do
      recursive true
      user args.user
      group args.group
    end

    template module_file do
      source 'module/main.iml.erb'
      cookbook 'intellij'
      user args.user
      group args.group
      variables(
        IS_MAVEN: maven_module?(args.path),
        SDK_NAME: args.sdk_name,
        SDK_TYPE: args.sdk_type,
        TYPE: args.type
      )
      action :create_if_missing
    end

    directory format(node['intellij']['project']['config_dir'], path: args.project_path) do
      recursive true
      user args.user
      group args.group
    end

    modules_file = format(node['intellij']['project']['config']['modules'], path: args.project_path)
    relative_modules_file = module_file.gsub(args.project_path, '$PROJECT_DIR$')

    template modules_file do
      source 'project/modules.xml'
      cookbook 'intellij'
      user args.user
      group args.group
      action :create_if_missing
    end

    xml_edit modules_file do
      parent '/project/component[@name="ProjectModuleManager"]/modules'
      target "/project/component[@name=\"ProjectModuleManager\"]/modules/module[@filepath=\"#{relative_modules_file}\"]"
      fragment "<module fileurl=\"file://#{relative_modules_file}\" filepath=\"#{relative_modules_file}\" />"
      action :append_if_missing
    end

    vcs_type = detect_vcs(args.path)

    if vcs_type
      vcs_file = format(node['intellij']['project']['config']['vcs'], path: args.project_path)

      template vcs_file do
        source 'project/vcs.xml'
        cookbook 'intellij'
        user args.user
        group args.group
        action :create_if_missing
      end

      xml_edit vcs_file do
        parent '/project/component[@name="VcsDirectoryMappings"]'
        target "/project/component[@name=\"VcsDirectoryMappings\"]/mapping[@directory=\"#{relative_modules_file}\"]"
        fragment "<mapping directory=\"#{relative_modules_file}\" vcs=\"#{vcs_type}\" />"
        only_if { vcs_type }
        action :append_if_missing
      end
    end

    if maven_module?(args.path)
      misc_file = format(node['intellij']['project']['config']['misc'], path: args.project_path)
      relative_pom = "#{args.path}/pom.xml".gsub(args.project_path, '$PROJECT_DIR$')

      template misc_file do
        source 'project/misc.xml'
        cookbook 'intellij'
        user args.user
        group args.group
        action :create_if_missing
      end

      create_missing_component('MavenProjectsManager', misc_file, '<option name="originalFiles"><list></list></option>')

      xml_edit misc_file do
        parent '/project/component[@name="MavenProjectsManager"]/option/list'
        target "/project/component[@name=\"MavenProjectsManager\"]/option/list/option[@value=\"#{relative_pom}\"]"
        fragment "<option value=\"#{relative_pom}\" />"
        action :append_if_missing
      end
    end
  end
end

action :delete do
  begin
    args = @new_resource

    module_file = generate_module_file_path(args.path, args.module_name)

    file module_file do
      action :delete
    end

    {
      modules: '/project/component[@name="ProjectModuleManager"]/modules/module[@filepath="%{relative_path}"]',
      vcs: '/project/component[@name="VcsDirectoryMappings"]/mapping[@directory="%{relative_path}"]',
    }.each do |type, target|
      modules_file = format(node['intellij']['project']['config'][type], path: args.project_path)
      relative_module = module_file.gsub(args.project_path, '$PROJECT_DIR$')

      xml_edit modules_file do
        target format(target, relative_path: relative_module)
        action :remove
        only_if { ::File.exist?(modules_file) }
      end
    end

    misc_file = format(node['intellij']['project']['config']['misc'], path: args.project_path)
    pom_file = "#{args.path}/pom.xml".gsub(args.project_path, '$PROJECT_DIR$')
    xml_edit misc_file do
      target format('/project/component[@name="MavenProjectsManager"]/option/list/option[@value="%{relative_path}"]', relative_path: pom_file)
      action :remove
      only_if { ::File.exist?(misc_file) }
    end

    directory args.path do
      action :delete
      recursive true
    end
  end
end
