require 'spec_helper'
require_relative '../../../../intellij/libraries/artifact'

describe 'intellij_artifact' do
  let(:chef_conf) { ChefSpec::SoloRunner.new step_into: %w(intellij_artifact) }
  let(:project_path) { '/home/user/myproject' }
  let(:user) { 'myuser' }
  let(:group) { 'mygroup' }

  describe 'testing the resource' do
    context 'creating the artifact' do
      let(:chef_run) { chef_conf.converge('test_intellij::artifact_create') }

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'creates the IntelliJ configuration' do
        expect(chef_run).to create_directory("#{project_path}/.idea/artifacts").with(user: user, group: group)
        expect(chef_run).to create_file("#{project_path}/.idea/artifacts/sample_maven_module_jar.xml").with(user: user, group: group)
      end
    end

    context 'removing the artifact' do
      let(:chef_run) { chef_conf.converge('test_intellij::artifact_delete') }

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'deletes the artifact' do
        expect(chef_run).to delete_file("#{project_path}/.idea/artifacts/sample_maven_module_jar.xml")
      end
    end
  end

  describe 'testing the artifact library' do
    before do
      class ArtifactLibrary
        include IntelliJ::Workspace::Artifact
      end
    end

    context 'generating artifact file name' do
      it 'generates the proper artifact file name' do
        artifact_name = 'some-artifact with.weird:symbols'
        expected_filename = 'some_artifact_with_weird_symbols'

        result = ArtifactLibrary.generate_artifact_filename(artifact_name)

        expect(result).to eq(expected_filename)
      end
    end

    context 'building the xml for the artifact' do
      let(:type) { 'jar' }
      let(:artifact_name) { 'some-artifact with.weird:symbols' }
      let(:root) { '<root/>' }

      it 'generates full artifact xml and sets up the relative output' do
        output = "#{project_path}/artifacts-output"
        expected_xml = <<-EOH
          <component name="ArtifactManager">
            <artifact type="#{type}" name="#{artifact_name}">
              <output-path>$PROJECT_DIR$/artifacts-output</output-path>
              #{root}
            </artifact>
          </component>
        EOH

        result = ArtifactLibrary.build_xml(artifact_name, type, output, root, project_path)

        # Remove useless spaces
        expected_xml.gsub!(/\s+/, ' ')
        result.gsub!(/\s+/, ' ')

        expect(result).to eq(expected_xml)
      end

      it 'sets up default output if it is not set' do
        expected_xml = <<-EOH
          <component name="ArtifactManager">
            <artifact type="#{type}" name="#{artifact_name}">
              <output-path>$PROJECT_DIR$/out/artifacts/some_artifact_with_weird_symbols</output-path>
              #{root}
            </artifact>
          </component>
        EOH

        result = ArtifactLibrary.build_xml(artifact_name, type, nil, root, project_path)

        # Remove useless spaces
        expected_xml.gsub!(/\s+/, ' ')
        result.gsub!(/\s+/, ' ')

        expect(result).to eq(expected_xml)
      end
    end

    context 'generating the artifact path' do
      it 'returns path to the artifact' do
        artifact_name = 'some-artifact with.weird:symbols'
        expected_path = "#{project_path}/.idea/artifacts/some_artifact_with_weird_symbols.xml"

        result = ArtifactLibrary.get_artifact_path(project_path, artifact_name)

        expect(result).to eq(expected_path)
      end
    end
  end
end
