require 'spec_helper'
require_relative '../../../../intellij/libraries/.helpers'
require_relative '../../../../intellij/libraries/package'

def mock_intellij_exists(version, edition, path)
  allow(File).to receive(:exist?).with(anything).and_call_original
  allow(File).to receive(:exist?).with("#{path}/product-info.json").and_return true
  allow(File).to receive(:exist?).with("#{path}/build.txt").and_return true
  allow(File).to receive(:read).with(anything).and_call_original
  allow(File).to receive(:read).with("#{path}/product-info.json").and_return "{\"version\": \"#{version}\", \"productCode\": \"#{edition}\"}"
end

def mock_intellij_not_exists(path)
  allow(File).to receive(:exist?).with(anything).and_call_original
  allow(File).to receive(:exist?).with("#{path}/product-info.json").and_return false
  allow(File).to receive(:exist?).with("#{path}/build.txt").and_return false
end

describe 'intellij_default' do
  let(:chef_conf) { ChefSpec::SoloRunner.new step_into: %w(intellij) }
  let(:intellij_path) { '/home/myuser/IntelliJ' }
  let(:user) { 'myuser' }
  let(:group) { 'mygroup' }

  describe 'testing the resource' do
    context 'installing the intellij' do
      let(:chef_run) { chef_conf.converge('test_intellij::intellij_install') }

      before do
        mock_intellij_not_exists(intellij_path)
      end

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'creates the IntelliJ configuration' do
        expect(chef_run).to create_directory(intellij_path).with(user: user, group: group)
      end

      it 'does install the IntelliJ' do
        expect(chef_run).to create_remote_file("#{Dir.tmpdir}/intellij.tar.gz").with(user: user, group: group)
        expect(chef_run).to_not extract_zipfile("#{Dir.tmpdir}/intellij.tar.gz")
        expect(chef_run).to extract_local_tar_extract("#{Dir.tmpdir}/intellij.tar.gz").with(user: user, group: group)
      end
    end

    context 'installing the intellij when it is already installed' do
      let(:chef_run) { chef_conf.converge('test_intellij::intellij_install') }

      before do
        mock_intellij_exists('2018.3.4', 'IC', intellij_path)
      end

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'prepares the IntelliJ installation' do
        expect(chef_run).to create_directory(intellij_path).with(user: user, group: group)
      end

      it { is_expected.to write_log("The IntelliJ community edition in #{intellij_path} is already installed in the 2018.3.4 version").with_level(:info) }

      it 'does not update the local IntelliJ' do
        expect(chef_run).to_not create_remote_file("#{Dir.tmpdir}/intellij.tar.gz")
        expect(chef_run).to_not extract_zipfile("#{Dir.tmpdir}/intellij.tar.gz")
        expect(chef_run).to_not extract_local_tar_extract("#{Dir.tmpdir}/intellij.tar.gz")
      end
    end

    context 'installing the intellij when it is already installed in the different version' do
      let(:chef_run) { chef_conf.converge('test_intellij::intellij_install') }

      before do
        mock_intellij_exists('2018.3.3', 'IU', intellij_path)
      end

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'prepares the IntelliJ installation' do
        expect(chef_run).to create_directory(intellij_path).with(user: user, group: group)
      end

      it { is_expected.to write_log("The IntelliJ in #{intellij_path} is already installed, but does not match requirements (edition: IC => IU, version: 2018.3.4 => 2018.3.3)").with_level(:warn) }

      it 'does not update the local IntelliJ' do
        expect(chef_run).to_not create_remote_file("#{Dir.tmpdir}/intellij.tar.gz")
        expect(chef_run).to_not extract_zipfile("#{Dir.tmpdir}/intellij.tar.gz")
        expect(chef_run).to_not extract_local_tar_extract("#{Dir.tmpdir}/intellij.tar.gz")
      end
    end

    context 'updating the intellij when there is no intellij installed' do
      let(:chef_run) { chef_conf.converge('test_intellij::intellij_update') }

      before do
        mock_intellij_not_exists(intellij_path)
      end

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'creates the IntelliJ configuration' do
        expect(chef_run).to create_directory(intellij_path).with(user: user, group: group)
      end

      it 'installs the IntelliJ' do
        expect(chef_run).to delete_directory(intellij_path)
        expect(chef_run).to install_intellij(intellij_path).with(user: user, group: group)
      end
    end

    context 'updating the intellij when it is already installed' do
      let(:chef_run) { chef_conf.converge('test_intellij::intellij_update') }

      before do
        mock_intellij_exists('2018.3.4', 'IC', intellij_path)
      end

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it { is_expected.to write_log("The IntelliJ community edition in #{intellij_path} is already installed in the 2018.3.4 version").with_level(:info) }

      it 'does not update the local IntelliJ' do
        expect(chef_run).to_not delete_directory(intellij_path)
        expect(chef_run).to_not install_intellij(intellij_path).with(user: user, group: group)
      end
    end

    context 'updating the intellij when it is already installed in the different version' do
      let(:chef_run) { chef_conf.converge('test_intellij::intellij_update') }

      before do
        mock_intellij_exists('2018.3.3', 'IU', intellij_path)
      end

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'prepares the IntelliJ installation' do
        expect(chef_run).to create_directory(intellij_path).with(user: user, group: group)
      end

      it { is_expected.to write_log("Removing the previous IntelliJ 2018.3.3 IU edition in #{intellij_path}").with_level(:info) }

      it 'updates the local intellij' do
        expect(chef_run).to delete_directory(intellij_path)
        expect(chef_run).to install_intellij(intellij_path).with(user: user, group: group)
      end
    end

    context 'deleting the intellij' do
      let(:chef_run) { chef_conf.converge('test_intellij::intellij_delete') }

      before do
        mock_intellij_exists('2018.3.3', 'IU', intellij_path)
      end

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it { is_expected.to write_log("Removing the previous IntelliJ 2018.3.3 IU edition in #{intellij_path}").with_level(:info) }

      it 'updates the local intellij' do
        expect(chef_run).to delete_directory(intellij_path)
      end
    end
  end

  describe 'testing the package library' do
    before do
      class PackageLibrary
        include IntelliJ::Install
      end

      context 'getting the info about local intellij' do
        it 'returns the local intellij info when it does exist' do
          mock_intellij_exists('2018.3.4', 'IC', intellij_path)

          expected_result = %w(2018.3.4 IU)

          result = PackageLibrary.get_local_intellij_info(intellij_path)

          expect(result).to eq(expected_result)
        end

        it 'returns nothing when the intellij does not exist' do
          mock_intellij_not_exists(intellij_path)

          expected_result = nil

          result = PackageLibrary.get_local_intellij_info(intellij_path)

          expect(result).to eq(expected_result)
        end
      end

      context 'checking if intellij exists' do
        it 'returns true when it does exist' do
          mock_intellij_exists('2018.3.4', 'IC', intellij_path)

          result = PackageLibrary.intellij_installed?(intellij_path)

          expect(result).to eq(true)
        end

        it 'returns false when it does not exist' do
          mock_intellij_not_exists(intellij_path)

          result = PackageLibrary.intellij_installed?(intellij_path)

          expect(result).to eq(false)
        end
      end
    end
  end
end
