require 'spec_helper'
require_relative '../../../../intellij/libraries/package'

describe 'intellij_module' do
  let(:chef_conf) { ChefSpec::SoloRunner.new step_into: %w(intellij_module) }
  let(:project_path) { '/home/user/myproject' }
  let(:module_path) { '/home/user/myproject/Module2' }
  let(:user) { 'myuser' }
  let(:group) { 'mygroup' }

  describe 'testing the resource' do
    context 'creating the module' do
      let(:chef_run) { chef_conf.converge('test_intellij::module_create') }

      before do
        allow(File).to receive(:exist?).with(anything).and_call_original
        allow(File).to receive(:exist?).with("#{module_path}/pom.xml").and_return true
        allow(File).to receive(:exist?).with("#{module_path}/.git").and_return true
        allow(File).to receive(:read).with(anything).and_call_original
        allow(File).to receive(:read).with("#{project_path}/.idea/misc.xml").and_return '<project/>'
      end

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'creates the module' do
        expect(chef_run).to create_directory(module_path).with(user: user, group: group)
        expect(chef_run).to create_if_missing_template("#{module_path}/sample-maven-module.iml").with(user: user, group: group)
      end

      it 'updates the project configuration' do
        expect(chef_run).to create_directory("#{project_path}/.idea").with(user: user, group: group)
        expect(chef_run).to create_if_missing_template("#{project_path}/.idea/modules.xml").with(user: user, group: group)
        expect(chef_run).to create_if_missing_template("#{project_path}/.idea/vcs.xml").with(user: user, group: group)
        expect(chef_run).to create_if_missing_template("#{project_path}/.idea/misc.xml").with(user: user, group: group)
        expect(chef_run).to append_if_missing_xml("#{project_path}/.idea/modules.xml")
          .with(target: '/project/component[@name="ProjectModuleManager"]/modules/module[@filepath="$PROJECT_DIR$/Module2/sample-maven-module.iml"]')
        expect(chef_run).to append_if_missing_xml("#{project_path}/.idea/vcs.xml")
          .with(target: '/project/component[@name="VcsDirectoryMappings"]/mapping[@directory="$PROJECT_DIR$/Module2/sample-maven-module.iml"]')
        expect(chef_run).to append_if_missing_xml("#{project_path}/.idea/misc.xml")
          .with(target: '/project/component[@name="MavenProjectsManager"]/option/list/option[@value="$PROJECT_DIR$/Module2/pom.xml"]')
      end
    end

    context 'deleting the module' do
      let(:chef_run) { chef_conf.converge('test_intellij::module_delete') }

      before do
        allow(File).to receive(:exist?).with(anything).and_call_original
        allow(File).to receive(:exist?).with("#{project_path}/.idea/modules.xml").and_return true
        allow(File).to receive(:exist?).with("#{project_path}/.idea/vcs.xml").and_return true
        allow(File).to receive(:exist?).with("#{project_path}/.idea/misc.xml").and_return true
      end

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'deletes the module' do
        expect(chef_run).to delete_directory(module_path)
      end

      it 'updates the project configuration' do
        expect(chef_run).to remove_xml("#{project_path}/.idea/modules.xml")
          .with(target: '/project/component[@name="ProjectModuleManager"]/modules/module[@filepath="$PROJECT_DIR$/Module2/sample-maven-module.iml"]')
        expect(chef_run).to remove_xml("#{project_path}/.idea/vcs.xml")
          .with(target: '/project/component[@name="VcsDirectoryMappings"]/mapping[@directory="$PROJECT_DIR$/Module2/sample-maven-module.iml"]')
        expect(chef_run).to remove_xml("#{project_path}/.idea/misc.xml")
          .with(target: '/project/component[@name="MavenProjectsManager"]/option/list/option[@value="$PROJECT_DIR$/Module2/pom.xml"]')
      end
    end
  end

  describe 'testing the package library' do
    before do
      class ModuleLibrary
        include IntelliJ::Module
      end

      context 'generating the module name' do
        it 'should use provided name' do
          name = 'sample-maven-module'
          expected_result = "#{module_path}/sample-maven-module.iml"

          result = PackageLibrary.generate_module_file_path(module_path, name)

          expect(result).to eq(expected_result)
        end

        it 'create a name from the directory if no name is provided' do
          name = nil
          expected_result = "#{module_path}/Module2.iml"

          result = PackageLibrary.generate_module_file_path(module_path, name)

          expect(result).to eq(expected_result)
        end
      end

      context 'checking if the module is maven module' do
        it 'returns true when it does exist' do
          allow(File).to receive(:exist?).with(anything).and_call_original
          allow(File).to receive(:exist?).with("#{module_path}/pom.xml").and_return true

          result = PackageLibrary.maven_module?(module_path)

          expect(result).to eq(true)
        end

        it 'returns false when it does not exist' do
          allow(File).to receive(:exist?).with(anything).and_call_original
          allow(File).to receive(:exist?).with("#{module_path}/pom.xml").and_return false

          result = PackageLibrary.maven_module?(module_path)

          expect(result).to eq(false)
        end
      end
    end
  end
end
