require 'spec_helper'
require_relative '../../../../intellij/libraries/project'

describe 'intellij_project' do
  let(:chef_conf) { ChefSpec::SoloRunner.new step_into: %w(intellij_project) }
  let(:project_path) { '/home/user/myproject' }
  let(:user) { 'myuser' }
  let(:group) { 'mygroup' }

  describe 'testing the resource' do
    context 'creating the project' do
      let(:chef_run) { chef_conf.converge('test_intellij::project_create') }

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'configures the IntelliJ project configuration' do
        expect(chef_run).to create_directory(project_path).with(user: user, group: group)
        expect(chef_run).to create_directory("#{project_path}/.idea").with(user: user, group: group)
        expect(chef_run).to create_file("#{project_path}/.idea/.name").with(user: user, group: group, content: 'My fantastic project')
        expect(chef_run).to create_template_if_missing("#{project_path}/.idea/misc.xml").with(user: user, group: group)
        expect(chef_run).to append_if_missing_xml("#{project_path}/.idea/misc.xml")
          .with(target: '/project/component[@name="ProjectRootManager"]')
      end
    end

    context 'deleting the project' do
      let(:chef_run) { chef_conf.converge('test_intellij::project_delete') }

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'deletes the IntelliJ run configuration' do
        expect(chef_run).to delete_directory(project_path)
      end
    end
  end

  describe 'testing the run configuration library' do
    let(:language_level) { 'JDK_10' }

    before do
      class ProjectLibrary
        include IntelliJ::Project
      end
    end

    context 'generating the SDK fragment' do
      it 'returns default xml for no SDK' do
        expected_xml = <<-EOH
          <component name=\"ProjectRootManager\" version=\"2\" languageLevel=\"#{language_level}\" default=\"false\"/>
        EOH

        result = ProjectLibrary.get_default_sdk_fragment(nil, nil, language_level)

        result.gsub!(/\s+/, ' ')
        expected_xml.gsub!(/\s+/, ' ').gsub!(/^\s+/, '').gsub!(/\s+$/, '')

        expect(result).to eq(expected_xml)
      end

      it 'returns xml for the JavaSDK' do
        sdk_name = 'JDK 10'
        sdk_type = :JavaSDK
        expected_xml = <<-EOH
         <component name="ProjectRootManager" version="2" languageLevel="#{language_level}" default="false"
            project-jdk-name="#{sdk_name}" project-jdk-type="#{sdk_type}">
           <output url="file://$PROJECT_DIR$/out" />
         </component>
        EOH

        result = ProjectLibrary.get_default_sdk_fragment(sdk_name, sdk_type, language_level)

        result.gsub!(/\s+/, ' ')
        expected_xml.gsub!(/\s+/, ' ')

        expect(result).to eq(expected_xml)
      end

      it 'returns xml for other SDKs' do
        sdk_name = 'Ruby 2.6'
        sdk_type = :RUBY_SDK
        expected_xml = <<-EOH
          <component name=\"ProjectRootManager\"
              version=\"2\" languageLevel=\"#{language_level}\" default=\"false\"
              project-jdk-name=\"#{sdk_name}\" project-jdk-type=\"#{sdk_type}\" />
        EOH

        result = ProjectLibrary.get_default_sdk_fragment(sdk_name, sdk_type, language_level)

        result.gsub!(/\s+/, ' ')
        expected_xml.gsub!(/\s+/, ' ').gsub!(/^\s+/, '').gsub!(/\s+$/, '')

        expect(result).to eq(expected_xml)
      end
    end
  end
end
