require 'spec_helper'
require_relative '../../../../intellij/libraries/run_configuration'

describe 'intellij_run_configuration' do
  let(:chef_conf) { ChefSpec::SoloRunner.new step_into: %w(intellij_run_configuration) }
  let(:project_path) { '/home/user/myproject' }
  let(:user) { 'myuser' }
  let(:group) { 'mygroup' }

  describe 'testing the resource' do
    let(:config_name) { 'Start simple batch script' }

    context 'creating the run configuration' do
      let(:chef_run) { chef_conf.converge('test_intellij::run_configuration_create') }

      before do
        allow(File).to receive(:read).with(anything).and_call_original
        allow(File).to receive(:read).with("#{project_path}/.idea/workspace.xml").and_return '<project version="4"></project>'
      end

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'configures the IntelliJ run configuration' do
        expect(chef_run).to create_directory("#{project_path}/.idea").with(user: user, group: group)
        expect(chef_run).to create_template_if_missing("#{project_path}/.idea/workspace.xml").with(user: user, group: group)
        expect(chef_run).to append_if_missing_xml("#{project_path}/.idea/workspace.xml")
          .with(target: "/project/component[@name=\"RunManager\"]/configuration[@name=\"#{config_name}\"]")
      end
    end

    context 'deleting the run configuration' do
      let(:chef_run) { chef_conf.converge('test_intellij::run_configuration_delete') }

      before do
        allow(File).to receive(:exist?).with(anything).and_call_original
        allow(File).to receive(:exist?).with("#{project_path}/.idea/workspace.xml").and_return true
      end

      it 'converges successfully' do
        expect { chef_run }.to_not raise_error
      end

      it 'deletes the IntelliJ run configuration' do
        expect(chef_run).to remove_xml("#{project_path}/.idea/workspace.xml")
          .with(target: "/project/component[@name=\"RunManager\"]/configuration[@name=\"#{config_name}\"]")
      end
    end
  end

  describe 'testing the run configuration library' do
    let(:config_name) { 'My name' }
    let(:type) { 'type' }
    let(:factory) { 'factory' }
    let(:body) { '<options/>' }

    before do
      class RunConfigLibrary
        include IntelliJ::Workspace::RunConfiguration
      end
    end

    context 'generating the xml for the run configuration' do
      it 'generates proper xml when no folder is specified' do
        folder = nil
        expected_xml = <<-EOH
           <configuration name="#{config_name}" type="#{type}" factory="#{factory}">
            #{body}
          </configuration>
        EOH
        result = RunConfigLibrary.generate_run_config_xml(config_name, type, factory, folder, body)

        expected_xml.gsub!(/\s+/, ' ')
        result.gsub!(/\s+/, ' ')

        expect(result).to eq(expected_xml)
      end

      it 'generates proper xml when a folder is specified' do
        folder = 'my folder'
        expected_xml = <<-EOH
           <configuration name="#{config_name}" type="#{type}" factory="#{factory}" folderName="#{folder}">
            #{body}
          </configuration>
        EOH
        result = RunConfigLibrary.generate_run_config_xml(config_name, type, factory, folder, body)

        expected_xml.gsub!(/\s+/, ' ')
        result.gsub!(/\s+/, ' ')

        expect(result).to eq(expected_xml)
      end
    end
  end
end
